#include "WiFi.h"
#include "SPIFFS.h"
#include "Plumbing.h"
#include "ESP32MotorControl.h" 	// https://github.com/JoaoLopesF/ESP32MotorControl

String LightsState;
String AutoState;
String Speed = "1";

int LightsPin1 = 25;
int LightsPin2 = 26;
int HornPin = 17;

int MotorA_IN1Pin = 27;
int MotorA_IN2Pin = 16;
int MotorB_IN1Pin = 12;
int MotorB_IN2Pin = 14;
int MotorsEnablePin = 13;

int Sonar_TriggerPin = 5;
int Sonar_EchoPin = 23;

ESP32MotorControl MotorControl = ESP32MotorControl();

void setup() {
    Serial.begin(115200);

    pinMode(BUILTIN_LED, OUTPUT);
    pinMode(LightsPin1, OUTPUT);
    pinMode(LightsPin2, OUTPUT);
    pinMode(HornPin, OUTPUT);
    pinMode(MotorsEnablePin, OUTPUT);

    pinMode(Sonar_TriggerPin, OUTPUT);
    pinMode(Sonar_EchoPin, INPUT);
    digitalWrite(Sonar_TriggerPin, LOW);

    digitalWrite(MotorsEnablePin, LOW);
    digitalWrite(HornPin, HIGH);

    MotorControl.attachMotors(MotorA_IN1Pin, MotorA_IN2Pin, MotorB_IN1Pin, MotorB_IN2Pin);

    MoveStop();

    InitializeFileSystem();
    InitializeWebServer();
    ConnetToWifi(); 
    
    AutoState = "OFF";
    LightsState = "OFF";
    digitalWrite(BUILTIN_LED, LOW);

    digitalWrite(MotorsEnablePin, HIGH);
}

void loop() {
    ServerLoop(PocessRequest);
    SonarLoop();
}

void PocessRequest(WiFiClient client, String header)
{
    if (RequestHas(header, "lights"))
    {
        ToggleLights();
        SendToClient(client, LightsState);
    }
    else if (RequestHas(header, "horn"))
    {
        Horn();
    }
    else if (RequestHas(header, "auto"))
    {
        ToggleAuto();
        SendToClient(client, AutoState);
    }
    else if (RequestHas(header, "speed"))
    {
        SetSpeed(header);
    }
    else if (RequestHas(header, "forward"))
    {
        MoveForward();
    }
    else if (RequestHas(header, "backward"))
    {
        MoveBackward();
    }
    else if (RequestHas(header, "left"))
    {
        TurnLeft();
    }
    else if (RequestHas(header, "right"))
    {
        TurnRight();
    }
    else if (RequestHas(header, "stop"))
    {
        MoveStop();
    }
}

void ToggleLights()
{
    if (LightsState == "OFF")
    {
        LightsState = "ON";
        digitalWrite(LightsPin1, HIGH);
        digitalWrite(LightsPin2, HIGH);
    }
    else
    {
        LightsState = "OFF";
        digitalWrite(LightsPin1, LOW);
        digitalWrite(LightsPin2, LOW);
    }
}

void Horn()
{
    digitalWrite(HornPin, LOW);
    delay(250);
    digitalWrite(HornPin, HIGH);
}

void ToggleAuto()
{
    if (AutoState == "OFF")
    {
        AutoState = "ON";
        MoveForward();
    }
    else
    {
        AutoState = "OFF";
        MoveStop();
    }
}

void SetSpeed(String header)
{
    int pos = header.indexOf("speed") + 5;
    Speed = header.substring(pos, pos + 1);
}

int GetDigitalSpeed()
{
    if (AutoState == "ON") return 60;

    int speed = map(Speed.toInt(), 0, 5, 40, 200);
    return speed;
}

void MoveForward()
{
    int digitalSpeed = GetDigitalSpeed();

    MotorControl.motorForward(0, digitalSpeed);
    MotorControl.motorForward(1, digitalSpeed);
}

void MoveBackward()
{
    int digitalSpeed = GetDigitalSpeed();

    MotorControl.motorReverse(0, digitalSpeed);
    MotorControl.motorReverse(1, digitalSpeed);
}

void MoveLeft()
{
    int digitalSpeed = GetDigitalSpeed();

    MotorControl.motorForward(0, digitalSpeed);
    MotorControl.motorStop(1);
}

void TurnLeft()
{
    int digitalSpeed = GetDigitalSpeed();

    MotorControl.motorForward(0, digitalSpeed);
    MotorControl.motorReverse(1, digitalSpeed);
}

void MoveRight()
{
    int digitalSpeed = GetDigitalSpeed();

    MotorControl.motorStop(0);
    MotorControl.motorForward(1, digitalSpeed);
}

void TurnRight()
{
    int digitalSpeed = GetDigitalSpeed();

    MotorControl.motorReverse(0, digitalSpeed);
    MotorControl.motorForward(1, digitalSpeed);
}

void MoveStop()
{
    MotorControl.motorsStop();
}

void SonarLoop()
{
    if (AutoState != "ON") return;

    digitalWrite(Sonar_TriggerPin, HIGH);
    delayMicroseconds(10);
    digitalWrite(Sonar_TriggerPin, LOW);
    long distance = pulseIn(Sonar_EchoPin, HIGH) / 59;
    delay(50);

    if (distance < 50)
    {
        MoveStop();
        delay(500);
        MoveBackward();
        delay(300);
        TurnRight();
        delay(500);
    }
    else
    {
        MoveForward();
    }
}

void Blink(int count)
{
    for (int i = 1; i <= count; i++)
    {
        digitalWrite(BUILTIN_LED, HIGH);
        delay(200);
        digitalWrite(BUILTIN_LED, LOW);
        delay(200);
    }
}
